/*
 * System Interface Library for games
 * Copyright (c) 2007-2014 Andrew Church <achurch@achurch.org>
 * Released under the GNU GPL version 3 or later; NO WARRANTY is provided.
 * See the file COPYING.txt for details.
 *
 * src/graphics.h: Internal header for graphics functionality.
 */

#ifndef SIL_SRC_GRAPHICS_H
#define SIL_SRC_GRAPHICS_H

#include "SIL/graphics.h"  // Include the public header.

/*************************************************************************/
/*************************************************************************/

/**
 * GRAPHICS_VERTEX_FORMAT_TYPE, GRAPHICS_VERTEX_FORMAT_OFFSET:  Macros to
 * extract the type or offset value from a vertex format entry constructed
 * by the GRAPHICS_VERTEX_FORMAT() macro.
 */
#define GRAPHICS_VERTEX_FORMAT_TYPE(format) \
    ((GraphicsVertexDataID)((uint32_t)(format) >> 16))
#define GRAPHICS_VERTEX_FORMAT_OFFSET(format) \
    ((uint32_t)(format) & 0xFFFF)

/*-----------------------------------------------------------------------*/

/**
 * graphics_init:  Initialize the graphics/rendering subsystem.  This does
 * _not_ prepare the display itself; graphics_set_display_mode() must be
 * called before any rendering operations are performed.
 *
 * It is invalid to call any other graphics/rendering functions except
 * graphics_cleanup() without first successfully calling graphics_init().
 *
 * [Return value]
 *     True on success, false on error.
 */
extern int graphics_init(void);

/**
 * graphics_cleanup:  Shut down the graphics/rendering subsystem, closing
 * the display if it is open.
 */
extern void graphics_cleanup(void);

/*************************************************************************/
/*************************************************************************/

#endif  // SIL_SRC_GRAPHICS_H

/*
 * Local variables:
 *   c-file-style: "stroustrup"
 *   c-file-offsets: ((case-label . *) (statement-case-intro . *))
 *   indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
 */
